
import java.util.Properties;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.util.Map;
import java.util.Map.Entry;
import java.util.Iterator;
import javax.naming.Name;
import javax.naming.Reference;
import javax.naming.Referenceable;
import javax.naming.BinaryRefAddr;
import javax.naming.directory.DirContext;
import javax.naming.directory.InitialDirContext;
import javax.naming.directory.BasicAttributes;

/**
 * The <code>JNDILibrary</code> class.
 *
 */
public
class JNDILibrary
implements Library, Referenceable {

  private
  Properties m_properties;

  /**
   * Constructs an instance of <code>JNDILibrary</code>.
   *
   * @param properties the properties containing JNDI information
   *
   */
  public
  JNDILibrary(Properties properties) {
    m_properties = properties;
  }

  /**
   * Gets the reference for the library.
   *
   * @returns the reference
   *
   */
  public
  Reference
  getReference() {
    Reference reference = null;
    try {
      // Store the properties as an array of bytes.
      ByteArrayOutputStream bytearrayoutputstream = new ByteArrayOutputStream();
      m_properties.store(bytearrayoutputstream, null);
      // Create a reference to the library.
      reference = new Reference(
        JNDILibrary.class.getName(),
        new BinaryRefAddr("properties", bytearrayoutputstream.toByteArray()),
        JNDILibraryFactory.class.getName(),
        null
      );
    }
    catch (Exception exception) {
      exception.printStackTrace();
    }
    return reference;
  }

  /**
   * Publishes an object in the library.
   *
   * @param object the object
   * @param stringName the name of the object
   * @param map a map containing the object's attributes
   *
   */
  public
  void
  publish(Object object, String stringName, Map map) {
    try {
      // Use the properties to establish an initial directory context.
	    DirContext dircontext = new InitialDirContext(m_properties);
      // Create an iterator contains all of the entries in the map.
      Iterator iterator = map.entrySet().iterator();
      // For each entry, create an attribute.
      BasicAttributes basicattributes = new BasicAttributes();
      while (iterator.hasNext()) {
        Map.Entry entry = (Map.Entry)iterator.next();
        basicattributes.put(entry.getKey().toString(), entry.getValue().toString());
      }
      // Bind the object to the specified name.
	    dircontext.rebind(stringName, object, basicattributes);
      // Close the context.
	    dircontext.close();
    }
    catch (Exception exception) {
      exception.printStackTrace();
    }
  }

  /**
   * Retrieves a copy of a published object from the library
   * and restores it if necessary.
   *
   * @param stringName the name of the object
   *
   * @returns the object, or null
   *
   */
  public
  Object
  retrieve(String stringName) {
    Object object = null;
    try {
      // Use the properties to establish an initial directory context.
	    DirContext dircontext = new InitialDirContext(m_properties);
      // Lookup the object using the specified name.
      object = dircontext.lookup(stringName);
      // Close the context.
	    dircontext.close();
    }
    catch (Exception exception) {
      exception.printStackTrace();
    }
    return object;
  }
}
